@@ -79,7 +79,7 @@ saveShinySaveState <- function(state) {
 
   # Look for a save.interface function. This will be defined by the hosting
   # environment if it supports bookmarking.
-  saveInterface <- getShinyOption("save.interface")
+  saveInterface <- getShinyOption("save.interface", default = NULL)
 
   if (is.null(saveInterface)) {
     if (inShinyServer()) {
@@ -217,6 +217,22 @@ RestoreContext <- R6Class("RestoreContext",
       self$dir <- NULL
     },
 
+    # Completely replace the state
+    set = function(active = FALSE, initErrorMessage = NULL, input = list(), values = list(), dir = NULL) {
+      # Validate all inputs
+      stopifnot(is.logical(active))
+      stopifnot(is.null(initErrorMessage) || is.character(initErrorMessage))
+      stopifnot(is.list(input))
+      stopifnot(is.list(values))
+      stopifnot(is.null(dir) || is.character(dir))
+
+      self$active <- active
+      self$initErrorMessage <- initErrorMessage
+      self$input <- RestoreInputSet$new(input)
+      self$values <- list2env2(values, parent = emptyenv())
+      self$dir <- dir
+    },
+
     # This should be called before a restore context is popped off the stack.
     flushPending = function() {
       self$input$flushPending()
@@ -280,7 +296,7 @@ RestoreContext <- R6Class("RestoreContext",
 
       # Look for a load.interface function. This will be defined by the hosting
       # environment if it supports bookmarking.
-      loadInterface <- getShinyOption("load.interface")
+      loadInterface <- getShinyOption("load.interface", default = NULL)
 
       if (is.null(loadInterface)) {
         if (inShinyServer()) {
@@ -453,7 +469,7 @@ hasCurrentRestoreContext <- function() {
   domain <- getDefaultReactiveDomain()
   if (!is.null(domain) && !is.null(domain$restoreContext))
     return(TRUE)
-  
+
   return(FALSE)
 }
 
@@ -479,7 +495,7 @@ getCurrentRestoreContext <- function() {
 #' Restore an input value
 #'
 #' This restores an input value from the current restore context. It should be
-#' called early on inside of input functions (like \code{\link{textInput}}).
+#' called early on inside of input functions (like [textInput()]).
 #'
 #' @param id Name of the input value to restore.
 #' @param default A default value to use, if there's no value to restore.
@@ -509,23 +525,23 @@ restoreInput <- function(id, default) {
 #' It typically is called from an observer. Note that this will not work in
 #' Internet Explorer 9 and below.
 #'
-#' For \code{mode = "push"}, only three updates are currently allowed:
+#' For `mode = "push"`, only three updates are currently allowed:
 #' \enumerate{
-#'   \item the query string (format: \code{?param1=val1&param2=val2})
-#'   \item the hash (format: \code{#hash})
+#'   \item the query string (format: `?param1=val1&param2=val2`)
+#'   \item the hash (format: `#hash`)
 #'   \item both the query string and the hash
-#'     (format: \code{?param1=val1&param2=val2#hash})
+#'     (format: `?param1=val1&param2=val2#hash`)
 #' }
 #'
-#' In other words, if \code{mode = "push"}, the \code{queryString} must start
-#' with either \code{?} or with \code{#}.
+#' In other words, if `mode = "push"`, the `queryString` must start
+#' with either `?` or with `#`.
 #'
 #' A technical curiosity: under the hood, this function is calling the HTML5
-#' history API (which is where the names for the \code{mode} argument come from).
-#' When \code{mode = "replace"}, the function called is
-#' \code{window.history.replaceState(null, null, queryString)}.
-#' When \code{mode = "push"}, the function called is
-#' \code{window.history.pushState(null, null, queryString)}.
+#' history API (which is where the names for the `mode` argument come from).
+#' When `mode = "replace"`, the function called is
+#' `window.history.replaceState(null, null, queryString)`.
+#' When `mode = "push"`, the function called is
+#' `window.history.pushState(null, null, queryString)`.
 #'
 #' @param queryString The new query string to show in the location bar.
 #' @param mode When the query string is updated, should the the current history
@@ -534,7 +550,7 @@ restoreInput <- function(id, default) {
 #'   context. The latter is useful if you want to navigate between states using
 #'   the browser's back and forward buttons. See Examples.
 #' @param session A Shiny session object.
-#' @seealso \code{\link{enableBookmarking}}, \code{\link{getQueryString}}
+#' @seealso [enableBookmarking()], [getQueryString()]
 #' @examples
 #' ## Only run these examples in interactive sessions
 #' if (interactive()) {
@@ -597,7 +613,7 @@ updateQueryString <- function(queryString, mode = c("replace", "push"),
 
 #' Create a button for bookmarking/sharing
 #'
-#' A \code{bookmarkButton} is a \code{\link{actionButton}} with a default label
+#' A `bookmarkButton` is a [actionButton()] with a default label
 #' that consists of a link icon and the text "Bookmark...". It is meant to be
 #' used for bookmarking state.
 #'
@@ -607,10 +623,10 @@ updateQueryString <- function(queryString, mode = c("replace", "push"),
 #' @param id An ID for the bookmark button. The only time it is necessary to set
 #'   the ID unless you have more than one bookmark button in your application.
 #'   If you specify an input ID, it should be excluded from bookmarking with
-#'   \code{\link{setBookmarkExclude}}, and you must create an observer that
+#'   [setBookmarkExclude()], and you must create an observer that
 #'   does the bookmarking when the button is pressed. See the examples below.
 #'
-#' @seealso \code{\link{enableBookmarking}} for more examples.
+#' @seealso [enableBookmarking()] for more examples.
 #'
 #' @examples
 #' ## Only run these examples in interactive sessions
@@ -660,10 +676,10 @@ bookmarkButton <- function(label = "Bookmark...",
 
 #' Generate a modal dialog that displays a URL
 #'
-#' The modal dialog generated by \code{urlModal} will display the URL in a
+#' The modal dialog generated by `urlModal` will display the URL in a
 #' textarea input, and the URL text will be selected so that it can be easily
-#' copied. The result from \code{urlModal} should be passed to the
-#' \code{\link{showModal}} function to display it in the browser.
+#' copied. The result from `urlModal` should be passed to the
+#' [showModal()] function to display it in the browser.
 #'
 #' @param url A URL to display in the dialog box.
 #' @param title A title for the dialog box.
@@ -719,8 +735,8 @@ urlModal <- function(url, title = "Bookmarked application link", subtitle = NULL
 
 #' Display a modal dialog for bookmarking
 #'
-#' This is a wrapper function for \code{\link{urlModal}} that is automatically
-#' called if an application is bookmarked but no other \code{\link{onBookmark}}
+#' This is a wrapper function for [urlModal()] that is automatically
+#' called if an application is bookmarked but no other [onBookmark()]
 #' callback was set. It displays a modal dialog with the bookmark URL, along
 #' with a subtitle that is appropriate for the type of bookmarking used ("url"
 #' or "server").
@@ -761,8 +777,8 @@ showBookmarkUrlModal <- function(url) {
 #' @details
 #'
 #' For restoring state to work properly, the UI must be a function that takes
-#' one argument, \code{request}. In most Shiny applications, the UI is not a
-#' function; it might have the form \code{fluidPage(....)}. Converting it to a
+#' one argument, `request`. In most Shiny applications, the UI is not a
+#' function; it might have the form `fluidPage(....)`. Converting it to a
 #' function is as simple as wrapping it in a function, as in
 #' \code{function(request) \{ fluidPage(....) \}}.
 #'
@@ -771,17 +787,17 @@ showBookmarkUrlModal <- function(url) {
 #' but not if the state is encoded in a URL.
 #'
 #' When bookmarking state, arbitrary values can be stored, by passing a function
-#' as the \code{onBookmark} argument. That function will be passed a
-#' \code{ShinySaveState} object. The \code{values} field of the object is a list
+#' as the `onBookmark` argument. That function will be passed a
+#' `ShinySaveState` object. The `values` field of the object is a list
 #' which can be manipulated to save extra information. Additionally, if the
-#' state is being saved on the server, and the \code{dir} field of that object
+#' state is being saved on the server, and the `dir` field of that object
 #' can be used to save extra information to files in that directory.
 #'
 #' For saved-to-server state, this is how the state directory is chosen:
 #' \itemize{
 #'   \item If running in a hosting environment such as Shiny Server or
 #'     Connect, the hosting environment will choose the directory.
-#'   \item If running an app in a directory with \code{\link{runApp}()}, the
+#'   \item If running an app in a directory with [runApp()], the
 #'     saved states will be saved in a subdirectory of the app called
 #'    shiny_bookmarks.
 #'   \item If running a Shiny app object that is generated from code (not run
@@ -789,22 +805,22 @@ showBookmarkUrlModal <- function(url) {
 #'     the current working directory called shiny_bookmarks.
 #' }
 #'
-#' When used with \code{\link{shinyApp}()}, this function must be called before
-#' \code{shinyApp()}, or in the \code{shinyApp()}'s \code{onStart} function. An
-#' alternative to calling the \code{enableBookmarking()} function is to use the
-#' \code{enableBookmarking} \emph{argument} for \code{shinyApp()}. See examples
+#' When used with [shinyApp()], this function must be called before
+#' `shinyApp()`, or in the `shinyApp()`'s `onStart` function. An
+#' alternative to calling the `enableBookmarking()` function is to use the
+#' `enableBookmarking` *argument* for `shinyApp()`. See examples
 #' below.
 #'
-#' @param store Either \code{"url"}, which encodes all of the relevant values in
-#'   a URL, \code{"server"}, which saves to disk on the server, or
-#'   \code{"disable"}, which disables any previously-enabled bookmarking.
+#' @param store Either `"url"`, which encodes all of the relevant values in
+#'   a URL, `"server"`, which saves to disk on the server, or
+#'   `"disable"`, which disables any previously-enabled bookmarking.
 #'
-#' @seealso \code{\link{onBookmark}}, \code{\link{onBookmarked}},
-#'   \code{\link{onRestore}}, and \code{\link{onRestored}} for registering
+#' @seealso [onBookmark()], [onBookmarked()],
+#'   [onRestore()], and [onRestored()] for registering
 #'   callback functions that are invoked when the state is bookmarked or
 #'   restored.
 #'
-#'   Also see \code{\link{updateQueryString}}.
+#'   Also see [updateQueryString()].
 #'
 #' @export
 #' @examples
@@ -983,7 +999,7 @@ enableBookmarking <- function(store = c("url", "server", "disable")) {
 #' @param names A character vector containing names of inputs to exclude from
 #'   bookmarking.
 #' @param session A shiny session object.
-#' @seealso \code{\link{enableBookmarking}} for examples.
+#' @seealso [enableBookmarking()] for examples.
 #' @export
 setBookmarkExclude <- function(names = character(0), session = getDefaultReactiveDomain()) {
   session$setBookmarkExclude(names)
@@ -998,17 +1014,17 @@ setBookmarkExclude <- function(names = character(0), session = getDefaultReactiv
 #' should be called within an application's server function.
 #'
 #' \itemize{
-#'   \item \code{onBookmark} registers a function that will be called just
+#'   \item `onBookmark` registers a function that will be called just
 #'     before Shiny bookmarks state.
-#'   \item \code{onBookmarked} registers a function that will be called just
+#'   \item `onBookmarked` registers a function that will be called just
 #'     after Shiny bookmarks state.
-#'   \item \code{onRestore} registers a function that will be called when a
+#'   \item `onRestore` registers a function that will be called when a
 #'     session is restored, after the server function executes, but before all
 #'     other reactives, observers and render functions are run.
-#'   \item \code{onRestored} registers a function that will be called after a
-#'     session is restored. This is similar to \code{onRestore}, but it will be
+#'   \item `onRestored` registers a function that will be called after a
+#'     session is restored. This is similar to `onRestore`, but it will be
 #'     called after all reactives, observers, and render functions run, and
-#'     after results are sent to the client browser. \code{onRestored}
+#'     after results are sent to the client browser. `onRestored`
 #'     callbacks can be useful for sending update messages to the client
 #'     browser.
 #' }
@@ -1019,25 +1035,25 @@ setBookmarkExclude <- function(names = character(0), session = getDefaultReactiv
 #' arguments to cancel the registration.
 #'
 #' The callback function that is passed to these functions should take one
-#' argument, typically named "state" (for \code{onBookmark}, \code{onRestore},
-#' and \code{onRestored}) or "url" (for \code{onBookmarked}).
+#' argument, typically named "state" (for `onBookmark`, `onRestore`,
+#' and `onRestored`) or "url" (for `onBookmarked`).
 #'
-#' For \code{onBookmark}, the state object has three relevant fields. The
-#' \code{values} field is an environment which can be used to save arbitrary
+#' For `onBookmark`, the state object has three relevant fields. The
+#' `values` field is an environment which can be used to save arbitrary
 #' values (see examples). If the state is being saved to disk (as opposed to
-#' being encoded in a URL), the \code{dir} field contains the name of a
+#' being encoded in a URL), the `dir` field contains the name of a
 #' directory which can be used to store extra files. Finally, the state object
-#' has an \code{input} field, which is simply the application's \code{input}
+#' has an `input` field, which is simply the application's `input`
 #' object. It can be read, but not modified.
 #'
-#' For \code{onRestore} and \code{onRestored}, the state object is a list. This
-#' list contains \code{input}, which is a named list of input values to restore,
-#' \code{values}, which is an environment containing arbitrary values that were
-#' saved in \code{onBookmark}, and \code{dir}, the name of the directory that
+#' For `onRestore` and `onRestored`, the state object is a list. This
+#' list contains `input`, which is a named list of input values to restore,
+#' `values`, which is an environment containing arbitrary values that were
+#' saved in `onBookmark`, and `dir`, the name of the directory that
 #' the state is being restored from, and which could have been used to save
 #' extra files.
 #'
-#' For \code{onBookmarked}, the callback function receives a string with the
+#' For `onBookmarked`, the callback function receives a string with the
 #' bookmark URL. This callback function should be used to display UI in the
 #' client browser with the bookmark URL. If no callback function is registered,
 #' then Shiny will by default display a modal dialog with the bookmark URL.
