@@ -10,25 +10,27 @@
 #'
 #' @param ... Elements to include within the page
 #' @param title The browser window title (defaults to the host URL of the page).
-#'   Can also be set as a side effect of the \code{\link{titlePanel}} function.
+#'   Can also be set as a side effect of the [titlePanel()] function.
 #' @param responsive This option is deprecated; it is no longer optional with
 #'   Bootstrap 3.
 #' @param theme Alternative Bootstrap stylesheet (normally a css file within the
 #'   www directory). For example, to use the theme located at
-#'   \code{www/bootstrap.css} you would use \code{theme = "bootstrap.css"}.
+#'   `www/bootstrap.css` you would use `theme = "bootstrap.css"`.
+#' @inheritParams bootstrapPage
 #'
-#' @return A UI defintion that can be passed to the \link{shinyUI} function.
+#' @return A UI defintion that can be passed to the [shinyUI] function.
 #'
-#' @details To create a fluid page use the \code{fluidPage} function and include
-#'   instances of \code{fluidRow} and \code{\link{column}} within it. As an
+#' @details To create a fluid page use the `fluidPage` function and include
+#'   instances of `fluidRow` and [column()] within it. As an
 #'   alternative to low-level row and column functions you can also use
-#'   higher-level layout functions like \code{\link{sidebarLayout}}.
+#'   higher-level layout functions like [sidebarLayout()].
 #'
-#' @note See the \href{http://shiny.rstudio.com/articles/layout-guide.html}{
-#'   Shiny-Application-Layout-Guide} for additional details on laying out fluid
+#' @note See the [
+#'   Shiny-Application-Layout-Guide](https://shiny.rstudio.com/articles/layout-guide.html) for additional details on laying out fluid
 #'   pages.
 #'
-#' @seealso \code{\link{column}}, \code{\link{sidebarLayout}}
+#' @family layout functions
+#' @seealso [column()]
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -86,11 +88,12 @@
 #' }
 #' @rdname fluidPage
 #' @export
-fluidPage <- function(..., title = NULL, responsive = NULL, theme = NULL) {
+fluidPage <- function(..., title = NULL, responsive = deprecated(), theme = NULL, lang = NULL) {
   bootstrapPage(div(class = "container-fluid", ...),
                 title = title,
                 responsive = responsive,
-                theme = theme)
+                theme = theme,
+                lang = lang)
 }
 
 
@@ -116,21 +119,24 @@ fluidRow <- function(...) {
 #'   Bootstrap 3.
 #' @param theme Alternative Bootstrap stylesheet (normally a css file within the
 #'   www directory). For example, to use the theme located at
-#'   \code{www/bootstrap.css} you would use \code{theme = "bootstrap.css"}.
+#'   `www/bootstrap.css` you would use `theme = "bootstrap.css"`.
+#' @inheritParams bootstrapPage
 #'
-#' @return A UI defintion that can be passed to the \link{shinyUI} function.
+#' @return A UI defintion that can be passed to the [shinyUI] function.
 #'
-#' @details To create a fixed page use the \code{fixedPage} function and include
-#'   instances of \code{fixedRow} and \code{\link{column}} within it. Note that
-#'   unlike \code{\link{fluidPage}}, fixed pages cannot make use of higher-level
-#'   layout functions like \code{sidebarLayout}, rather, all layout must be done
-#'   with \code{fixedRow} and \code{column}.
+#' @details To create a fixed page use the `fixedPage` function and include
+#'   instances of `fixedRow` and [column()] within it. Note that
+#'   unlike [fluidPage()], fixed pages cannot make use of higher-level
+#'   layout functions like `sidebarLayout`, rather, all layout must be done
+#'   with `fixedRow` and `column`.
 #'
-#' @note See the \href{http://shiny.rstudio.com/articles/layout-guide.html}{
-#'   Shiny Application Layout Guide} for additional details on laying out fixed
+#' @note See the [
+#'   Shiny Application Layout Guide](https://shiny.rstudio.com/articles/layout-guide.html) for additional details on laying out fixed
 #'   pages.
 #'
-#' @seealso \code{\link{column}}
+#' @family layout functions
+#'
+#' @seealso [column()]
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -153,11 +159,12 @@ fluidRow <- function(...) {
 #'
 #' @rdname fixedPage
 #' @export
-fixedPage <- function(..., title = NULL, responsive = NULL, theme = NULL) {
+fixedPage <- function(..., title = NULL, responsive = deprecated(), theme = NULL, lang = NULL) {
   bootstrapPage(div(class = "container", ...),
                 title = title,
                 responsive = responsive,
-                theme = theme)
+                theme = theme,
+                lang = lang)
 }
 
 #' @rdname fixedPage
@@ -169,8 +176,8 @@ fixedRow <- function(...) {
 
 #' Create a column within a UI definition
 #'
-#' Create a column for use within a  \code{\link{fluidRow}} or
-#' \code{\link{fixedRow}}
+#' Create a column for use within a  [fluidRow()] or
+#' [fixedRow()]
 #'
 #' @param width The grid width of the column (must be between 1 and 12)
 #' @param ... Elements to include within the column
@@ -178,10 +185,10 @@ fixedRow <- function(...) {
 #'   previous column.
 #'
 #' @return A column that can be included within a
-#'   \code{\link{fluidRow}} or \code{\link{fixedRow}}.
+#'   [fluidRow()] or [fixedRow()].
 #'
 #'
-#' @seealso \code{\link{fluidRow}}, \code{\link{fixedRow}}.
+#' @seealso [fluidRow()], [fixedRow()].
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -228,8 +235,12 @@ column <- function(width, ..., offset = 0) {
     stop("column width must be between 1 and 12")
 
   colClass <- paste0("col-sm-", width)
-  if (offset > 0)
-    colClass <- paste0(colClass, " col-sm-offset-", offset)
+  if (offset > 0) {
+    # offset-md-x is for bootstrap 4 forward compat
+    # (every size tier has been bumped up one level)
+    # https://github.com/twbs/bootstrap/blob/74b8fe7/docs/4.3/migration/index.html#L659
+    colClass <- paste0(colClass, " offset-md-", offset, " col-sm-offset-", offset)
+  }
   div(class = colClass, ...)
 }
 
@@ -240,10 +251,9 @@ column <- function(width, ..., offset = 0) {
 #' @param windowTitle The title that should be displayed by the browser window.
 #'
 #' @details Calling this function has the side effect of including a
-#'   \code{title} tag within the head. You can also specify a page title
+#'   `title` tag within the head. You can also specify a page title
 #'   explicitly using the `title` parameter of the top-level page function.
 #'
-#'
 #' @examples
 #' ## Only run examples in interactive R sessions
 #' if (interactive()) {
@@ -263,16 +273,23 @@ titlePanel <- function(title, windowTitle=title) {
 
 #' Layout a sidebar and main area
 #'
-#' Create a layout with a sidebar and main area. The sidebar is displayed with a
-#' distinct background color and typically contains input controls. The main
+#' Create a layout (`sidebarLayout()`) with a sidebar (`sidebarPanel()`) and
+#' main area (`mainPanel()`). The sidebar is displayed with a distinct
+#' background color and typically contains input controls. The main
 #' area occupies 2/3 of the horizontal width and typically contains outputs.
 #'
-#' @param sidebarPanel The \link{sidebarPanel} containing input controls
-#' @param mainPanel The \link{mainPanel} containing outputs
+#' @param sidebarPanel The `sidebarPanel()` containing input controls.
+#' @param mainPanel The `mainPanel()` containing outputs.
 #' @param position The position of the sidebar relative to the main area ("left"
-#'   or "right")
-#' @param fluid \code{TRUE} to use fluid layout; \code{FALSE} to use fixed
+#'   or "right").
+#' @param fluid `TRUE` to use fluid layout; `FALSE` to use fixed
 #'   layout.
+#' @param width The width of the sidebar and main panel. By default, the
+#'   sidebar takes up 1/3 of the width, and the main panel 2/3. The total
+#'   width must be 12 or less.
+#' @param ... Output elements to include in the sidebar/main panel.
+#'
+#' @family layout functions
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -337,16 +354,38 @@ sidebarLayout <- function(sidebarPanel,
     fixedRow(firstPanel, secondPanel)
 }
 
+#' @export
+#' @rdname sidebarLayout
+sidebarPanel <- function(..., width = 4) {
+  div(class=paste0("col-sm-", width),
+    tags$form(class="well",
+      # A11y semantic landmark for sidebar
+      role="complementary",
+      ...
+    )
+  )
+}
+
+#' @export
+#' @rdname sidebarLayout
+mainPanel <- function(..., width = 8) {
+  div(class=paste0("col-sm-", width),
+    # A11y semantic landmark for main region
+    role="main",
+    ...
+  )
+}
+
 #' Lay out UI elements vertically
 #'
 #' Create a container that includes one or more rows of content (each element
 #' passed to the container will appear on it's own line in the UI)
 #'
 #' @param ... Elements to include within the container
-#' @param fluid \code{TRUE} to use fluid layout; \code{FALSE} to use fixed
+#' @param fluid `TRUE` to use fluid layout; `FALSE` to use fixed
 #'   layout.
 #'
-#' @seealso \code{\link{fluidPage}}, \code{\link{flowLayout}}
+#' @family layout functions
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -377,14 +416,14 @@ verticalLayout <- function(..., fluid = TRUE) {
 #' Lays out elements in a left-to-right, top-to-bottom arrangement. The elements
 #' on a given row will be top-aligned with each other. This layout will not work
 #' well with elements that have a percentage-based width (e.g.
-#' \code{\link{plotOutput}} at its default setting of \code{width = "100\%"}).
+#' [plotOutput()] at its default setting of `width = "100%"`).
 #'
 #' @param ... Unnamed arguments will become child elements of the layout. Named
 #'   arguments will become HTML attributes on the outermost tag.
 #' @param cellArgs Any additional attributes that should be used for each cell
 #'   of the layout.
 #'
-#' @seealso \code{\link{verticalLayout}}
+#' @family layout functions
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -401,7 +440,7 @@ verticalLayout <- function(..., fluid = TRUE) {
 flowLayout <- function(..., cellArgs = list()) {
 
   children <- list(...)
-  childIdx <- !nzchar(names(children) %OR% character(length(children)))
+  childIdx <- !nzchar(names(children) %||% character(length(children)))
   attribs <- children[!childIdx]
   children <- children[childIdx]
 
@@ -415,7 +454,7 @@ flowLayout <- function(..., cellArgs = list()) {
 
 #' Input panel
 #'
-#' A \code{\link{flowLayout}} with a grey border and light grey background,
+#' A [flowLayout()] with a grey border and light grey background,
 #' suitable for wrapping inputs.
 #'
 #' @param ... Input controls or other HTML elements.
@@ -435,11 +474,13 @@ inputPanel <- function(...) {
 #'   arguments will become HTML attributes on the outermost tag.
 #' @param cellWidths Character or numeric vector indicating the widths of the
 #'   individual cells. Recycling will be used if needed. Character values will
-#'   be interpreted as CSS lengths (see \code{\link{validateCssUnit}}), numeric
+#'   be interpreted as CSS lengths (see [validateCssUnit()]), numeric
 #'   values as pixels.
 #' @param cellArgs Any additional attributes that should be used for each cell
 #'   of the layout.
 #'
+#' @family layout functions
+#'
 #' @examples
 #' ## Only run examples in interactive R sessions
 #' if (interactive()) {
@@ -482,7 +523,7 @@ inputPanel <- function(...) {
 splitLayout <- function(..., cellWidths = NULL, cellArgs = list()) {
 
   children <- list(...)
-  childIdx <- !nzchar(names(children) %OR% character(length(children)))
+  childIdx <- !nzchar(names(children) %||% character(length(children)))
   attribs <- children[!childIdx]
   children <- children[childIdx]
   count <- length(children)
@@ -509,41 +550,41 @@ splitLayout <- function(..., cellWidths = NULL, cellArgs = list()) {
 #'
 #' Creates row and column layouts with proportionally-sized cells, using the
 #' Flex Box layout model of CSS3. These can be nested to create arbitrary
-#' proportional-grid layouts. \strong{Warning:} Flex Box is not well supported
+#' proportional-grid layouts. **Warning:** Flex Box is not well supported
 #' by Internet Explorer, so these functions should only be used where modern
 #' browsers can be assumed.
 #'
-#' @details If you try to use \code{fillRow} and \code{fillCol} inside of other
-#'   Shiny containers, such as \code{\link{sidebarLayout}},
-#'   \code{\link{navbarPage}}, or even \code{tags$div}, you will probably find
-#'   that they will not appear. This is due to \code{fillRow} and \code{fillCol}
-#'   defaulting to \code{height="100\%"}, which will only work inside of
+#' @details If you try to use `fillRow` and `fillCol` inside of other
+#'   Shiny containers, such as [sidebarLayout()],
+#'   [navbarPage()], or even `tags$div`, you will probably find
+#'   that they will not appear. This is due to `fillRow` and `fillCol`
+#'   defaulting to `height="100%"`, which will only work inside of
 #'   containers that have determined their own size (rather than shrinking to
 #'   the size of their contents, as is usually the case in HTML).
 #'
 #'   To avoid this problem, you have two options:
 #'   \itemize{
-#'     \item only use \code{fillRow}/\code{fillCol} inside of \code{fillPage},
-#'       \code{fillRow}, or \code{fillCol}
-#'     \item provide an explicit \code{height} argument to
-#'       \code{fillRow}/\code{fillCol}
+#'     \item only use `fillRow`/`fillCol` inside of `fillPage`,
+#'       `fillRow`, or `fillCol`
+#'     \item provide an explicit `height` argument to
+#'       `fillRow`/`fillCol`
 #'   }
 #'
 #' @param ... UI objects to put in each row/column cell; each argument will
 #'   occupy a single cell. (To put multiple items in a single cell, you can use
-#'   \code{\link{tagList}} or \code{\link{div}} to combine them.) Named
-#'   arguments will be used as attributes on the \code{div} element that
+#'   [tagList()] or [div()] to combine them.) Named
+#'   arguments will be used as attributes on the `div` element that
 #'   encapsulates the row/column.
 #' @param flex Determines how space should be distributed to the cells. Can be a
-#'   single value like \code{1} or \code{2} to evenly distribute the available
+#'   single value like `1` or `2` to evenly distribute the available
 #'   space; or use a vector of numbers to specify the proportions. For example,
-#'   \code{flex = c(2, 3)} would cause the space to be split 40\%/60\% between
+#'   `flex = c(2, 3)` would cause the space to be split 40%/60% between
 #'   two cells. NA values will cause the corresponding cell to be sized
 #'   according to its contents (without growing or shrinking).
 #' @param width,height The total amount of width and height to use for the
-#'   entire row/column. For the default height of \code{"100\%"} to be
-#'   effective, the parent must be \code{fillPage}, another
-#'   \code{fillRow}/\code{fillCol}, or some other HTML element whose height is
+#'   entire row/column. For the default height of `"100%"` to be
+#'   effective, the parent must be `fillPage`, another
+#'   `fillRow`/`fillCol`, or some other HTML element whose height is
 #'   not determined by the height of its contents.
 #'
 #' @examples
@@ -660,37 +701,3 @@ flexfill <- function(..., direction, flex, width = width, height = height) {
   )
   do.call(tags$div, c(attrs, divArgs))
 }
-
-css <- function(..., collapse_ = "") {
-  props <- list(...)
-  if (length(props) == 0) {
-    return("")
-  }
-
-  if (is.null(names(props)) || any(names(props) == "")) {
-    stop("cssList expects all arguments to be named")
-  }
-
-  # Necessary to make factors show up as level names, not numbers
-  props[] <- lapply(props, paste, collapse = " ")
-
-  # Drop null args
-  props <- props[!sapply(props, empty)]
-  if (length(props) == 0) {
-    return("")
-  }
-
-  # Replace all '.' and '_' in property names to '-'
-  names(props) <- gsub("[._]", "-", tolower(gsub("([A-Z])", "-\\1", names(props))))
-
-  # Create "!important" suffix for each property whose name ends with !, then
-  # remove the ! from the property name
-  important <- ifelse(grepl("!$", names(props), perl = TRUE), " !important", "")
-  names(props) <- sub("!$", "", names(props), perl = TRUE)
-
-  paste0(names(props), ":", props, important, ";", collapse = collapse_)
-}
-
-empty <- function(x) {
-  length(x) == 0 || (is.character(x) && !any(nzchar(x)))
-}
