@@ -3,9 +3,9 @@
 #' Advanced (borderline internal) functions for capturing, printing, and
 #' manipulating stack traces.
 #'
-#' @return \code{printError} and \code{printStackTrace} return
-#'   \code{invisible()}. The other functions pass through the results of
-#'   \code{expr}.
+#' @return `printError` and `printStackTrace` return
+#'   `invisible()`. The other functions pass through the results of
+#'   `expr`.
 #'
 #' @examples
 #' # Keeps tryCatch and withVisible related calls off the
@@ -106,17 +106,17 @@ getCallCategories <- function(calls) {
   }, character(1))
 }
 
-#' @details \code{captureStackTraces} runs the given \code{expr} and if any
-#'   \emph{uncaught} errors occur, annotates them with stack trace info for use
-#'   by \code{printError} and \code{printStackTrace}. It is not necessary to use
-#'   \code{captureStackTraces} around the same expression as
-#'   \code{withLogErrors}, as the latter includes a call to the former. Note
-#'   that if \code{expr} contains calls (either directly or indirectly) to
-#'   \code{try}, or \code{tryCatch} with an error handler, stack traces therein
-#'   cannot be captured unless another \code{captureStackTraces} call is
-#'   inserted in the interior of the \code{try} or \code{tryCatch}. This is
+#' @details `captureStackTraces` runs the given `expr` and if any
+#'   *uncaught* errors occur, annotates them with stack trace info for use
+#'   by `printError` and `printStackTrace`. It is not necessary to use
+#'   `captureStackTraces` around the same expression as
+#'   `withLogErrors`, as the latter includes a call to the former. Note
+#'   that if `expr` contains calls (either directly or indirectly) to
+#'   `try`, or `tryCatch` with an error handler, stack traces therein
+#'   cannot be captured unless another `captureStackTraces` call is
+#'   inserted in the interior of the `try` or `tryCatch`. This is
 #'   because these calls catch the error and prevent it from traveling up to the
-#'   condition handler installed by \code{captureStackTraces}.
+#'   condition handler installed by `captureStackTraces`.
 #'
 #' @param expr The expression to wrap.
 #' @rdname stacktrace
@@ -133,7 +133,7 @@ captureStackTraces <- function(expr) {
 createStackTracePromiseDomain <- function() {
   # These are actually stateless, we wouldn't have to create a new one each time
   # if we didn't want to. They're pretty cheap though.
-  
+
   d <- promises::new_promise_domain(
     wrapOnFulfilled = function(onFulfilled) {
       force(onFulfilled)
@@ -209,15 +209,15 @@ doCaptureStack <- function(e) {
   stop(e)
 }
 
-#' @details \code{withLogErrors} captures stack traces and logs errors that
-#'   occur in \code{expr}, but does allow errors to propagate beyond this point
+#' @details `withLogErrors` captures stack traces and logs errors that
+#'   occur in `expr`, but does allow errors to propagate beyond this point
 #'   (i.e. it doesn't catch the error). The same caveats that apply to
-#'   \code{captureStackTraces} with regard to \code{try}/\code{tryCatch} apply
-#'   to \code{withLogErrors}.
+#'   `captureStackTraces` with regard to `try`/`tryCatch` apply
+#'   to `withLogErrors`.
 #' @rdname stacktrace
 #' @export
 withLogErrors <- function(expr,
-  full = getOption("shiny.fullstacktrace", FALSE),
+  full = get_devmode_option("shiny.fullstacktrace", FALSE),
   offset = getOption("shiny.stacktraceoffset", TRUE)) {
 
   withCallingHandlers(
@@ -247,51 +247,51 @@ withLogErrors <- function(expr,
   )
 }
 
-#' @details \code{printError} prints the error and stack trace (if any) using
-#'   \code{warning(immediate.=TRUE)}. \code{printStackTrace} prints the stack
+#' @details `printError` prints the error and stack trace (if any) using
+#'   `warning(immediate.=TRUE)`. `printStackTrace` prints the stack
 #'   trace only.
 #'
 #' @param cond An condition object (generally, an error).
-#' @param full If \code{TRUE}, then every element of \code{sys.calls()} will be
-#'   included in the stack trace. By default (\code{FALSE}), calls that Shiny
+#' @param full If `TRUE`, then every element of `sys.calls()` will be
+#'   included in the stack trace. By default (`FALSE`), calls that Shiny
 #'   deems uninteresting will be hidden.
-#' @param offset If \code{TRUE} (the default), srcrefs will be reassigned from
+#' @param offset If `TRUE` (the default), srcrefs will be reassigned from
 #'   the calls they originated from, to the destinations of those calls. If
 #'   you're used to stack traces from other languages, this feels more
 #'   intuitive, as the definition of the function indicated in the call and the
-#'   location specified by the srcref match up. If \code{FALSE}, srcrefs will be
+#'   location specified by the srcref match up. If `FALSE`, srcrefs will be
 #'   left alone (traditional R treatment where the srcref is of the callsite).
 #' @rdname stacktrace
 #' @export
 printError <- function(cond,
-  full = getOption("shiny.fullstacktrace", FALSE),
+  full = get_devmode_option("shiny.fullstacktrace", FALSE),
   offset = getOption("shiny.stacktraceoffset", TRUE)) {
-  
-  warning(call. = FALSE, immediate. = TRUE, sprintf("Error in %s: %s", 
+
+  warning(call. = FALSE, immediate. = TRUE, sprintf("Error in %s: %s",
     getCallNames(list(conditionCall(cond))), conditionMessage(cond)))
-  
+
   printStackTrace(cond, full = full, offset = offset)
 }
 
 #' @rdname stacktrace
 #' @export
 printStackTrace <- function(cond,
-  full = getOption("shiny.fullstacktrace", FALSE),
+  full = get_devmode_option("shiny.fullstacktrace", FALSE),
   offset = getOption("shiny.stacktraceoffset", TRUE)) {
 
   should_drop <- !full
   should_strip <- !full
   should_prune <- !full
-  
+
   stackTraceCalls <- c(
     attr(cond, "deep.stack.trace", exact = TRUE),
     list(attr(cond, "stack.trace", exact = TRUE))
   )
-  
+
   stackTraceParents <- lapply(stackTraceCalls, attr, which = "parents", exact = TRUE)
   stackTraceCallNames <- lapply(stackTraceCalls, getCallNames)
   stackTraceCalls <- lapply(stackTraceCalls, offsetSrcrefs, offset = offset)
-  
+
   # Use dropTrivialFrames logic to remove trailing bits (.handleSimpleError, h)
   if (should_drop) {
     # toKeep is a list of logical vectors, of which elements (stack frames) to keep
@@ -301,7 +301,7 @@ printStackTrace <- function(cond,
     stackTraceCallNames <- mapply(stackTraceCallNames, FUN = `[`, toKeep, SIMPLIFY = FALSE)
     stackTraceParents <- mapply(stackTraceParents, FUN = `[`, toKeep, SIMPLIFY = FALSE)
   }
-  
+
   delayedAssign("all_true", {
     # List of logical vectors that are all TRUE, the same shape as
     # stackTraceCallNames. Delay the evaluation so we don't create it unless
@@ -310,7 +310,7 @@ printStackTrace <- function(cond,
       rep_len(TRUE, length(st))
     })
   })
-  
+
   # stripStackTraces and lapply(stackTraceParents, pruneStackTrace) return lists
   # of logical vectors. Use mapply(FUN = `&`) to boolean-and each pair of the
   # logical vectors.
@@ -320,7 +320,7 @@ printStackTrace <- function(cond,
     FUN = `&`,
     SIMPLIFY = FALSE
   )
-  
+
   dfs <- mapply(seq_along(stackTraceCalls), rev(stackTraceCalls), rev(stackTraceCallNames), rev(toShow), FUN = function(i, calls, nms, index) {
     st <- data.frame(
       num = rev(which(index)),
@@ -329,7 +329,7 @@ printStackTrace <- function(cond,
       category = rev(getCallCategories(calls[index])),
       stringsAsFactors = FALSE
     )
-    
+
     if (i != 1) {
       message("From earlier call:")
     }
@@ -357,25 +357,26 @@ printStackTrace <- function(cond,
 
     st
   }, SIMPLIFY = FALSE)
-  
+
   invisible()
 }
 
-#' @details \code{extractStackTrace} takes a list of calls (e.g. as returned
-#'   from \code{conditionStackTrace(cond)}) and returns a data frame with one
-#'   row for each stack frame and the columns \code{num} (stack frame number),
-#'   \code{call} (a function name or similar), and \code{loc} (source file path
+#' @details `extractStackTrace` takes a list of calls (e.g. as returned
+#'   from `conditionStackTrace(cond)`) and returns a data frame with one
+#'   row for each stack frame and the columns `num` (stack frame number),
+#'   `call` (a function name or similar), and `loc` (source file path
 #'   and line number, if available). It was deprecated after shiny 1.0.5 because
 #'   it doesn't support deep stack traces.
 #' @rdname stacktrace
 #' @export
 extractStackTrace <- function(calls,
-  full = getOption("shiny.fullstacktrace", FALSE),
+  full = get_devmode_option("shiny.fullstacktrace", FALSE),
   offset = getOption("shiny.stacktraceoffset", TRUE)) {
-  
-  shinyDeprecated(NULL,
-    "extractStackTrace is deprecated. Please contact the Shiny team if you were using this functionality.",
-    version = "1.0.5")
+
+  shinyDeprecated(
+    "1.0.5", "extractStackTrace()",
+    details = "Please contact the Shiny team if you were using this functionality."
+  )
 
   srcrefs <- getSrcRefs(calls)
   if (offset) {
@@ -459,19 +460,19 @@ stripOneStackTrace <- function(stackTrace, truncateFloor, startingScore) {
       prefix <- rep_len(FALSE, indexOfFloor)
     }
   }
-  
+
   if (length(stackTrace) == 0) {
     return(list(score = startingScore, character(0)))
   }
-  
+
   score <- rep.int(0L, length(stackTrace))
   score[stackTrace == "..stacktraceon.."] <- 1L
   score[stackTrace == "..stacktraceoff.."] <- -1L
   score <- startingScore + cumsum(score)
-  
+
   toShow <- score > 0 & !(stackTrace %in% c("..stacktraceon..", "..stacktraceoff..", "..stacktracefloor.."))
-  
-  
+
+
   list(score = utils::tail(score, 1), trace = c(prefix, toShow))
 }
 
@@ -486,11 +487,11 @@ pruneStackTrace <- function(parents) {
   # sufficient; we also need to drop nodes that are the last child, but one of
   # their ancestors is not.
   is_dupe <- duplicated(parents, fromLast = TRUE)
-  
+
   # The index of the most recently seen node that was actually kept instead of
   # dropped.
   current_node <- 0
-  
+
   # Loop over the parent indices. Anything that is not parented by current_node
   # (a.k.a. last-known-good node), or is a dupe, can be discarded. Anything that
   # is kept becomes the new current_node.
@@ -502,7 +503,7 @@ pruneStackTrace <- function(parents) {
       FALSE
     }
   }, FUN.VALUE = logical(1))
-  
+
   include
 }
 
@@ -515,7 +516,7 @@ dropTrivialFrames <- function(callnames) {
   # What's the last that *didn't* match stop/.handleSimpleError/h?
   lastGoodCall <- max(which(!hideable))
   toRemove <- length(callnames) - lastGoodCall
-  
+
   c(
     rep_len(TRUE, length(callnames) - toRemove),
     rep_len(FALSE, toRemove)
@@ -530,27 +531,28 @@ offsetSrcrefs <- function(calls, offset = TRUE) {
     # E.g. for "foo [bar.R:10]", line 10 of bar.R will be part of
     # the definition of foo().
     srcrefs <- c(utils::tail(srcrefs, -1), list(NULL))
-    
+
     calls <- setSrcRefs(calls, srcrefs)
   }
-  
+
   calls
 }
 
-#' @details \code{formatStackTrace} is similar to \code{extractStackTrace}, but
+#' @details `formatStackTrace` is similar to `extractStackTrace`, but
 #'   it returns a preformatted character vector instead of a data frame. It was
 #'   deprecated after shiny 1.0.5 because it doesn't support deep stack traces.
 #' @param indent A string to prefix every line of the stack trace.
 #' @rdname stacktrace
 #' @export
 formatStackTrace <- function(calls, indent = "    ",
-  full = getOption("shiny.fullstacktrace", FALSE),
+  full = get_devmode_option("shiny.fullstacktrace", FALSE),
   offset = getOption("shiny.stacktraceoffset", TRUE)) {
 
-  shinyDeprecated(NULL,
-    "extractStackTrace is deprecated. Please contact the Shiny team if you were using this functionality.",
-    version = "1.0.5")
-  
+  shinyDeprecated(
+    "1.0.5", "formatStackTrace()",
+    details = "Please contact the Shiny team if you were using this functionality."
+  )
+
   st <- extractStackTrace(calls, full = full, offset = offset)
   if (nrow(st) == 0) {
     return(character(0))
@@ -588,11 +590,11 @@ stripStackTrace <- function(cond) {
   conditionStackTrace(cond) <- NULL
 }
 
-#' @details \code{conditionStackTrace} and \code{conditionStackTrace<-} are
+#' @details `conditionStackTrace` and `conditionStackTrace<-` are
 #'   accessor functions for getting/setting stack traces on conditions.
 #'
 #' @param cond A condition that may have previously been annotated by
-#'   \code{captureStackTraces} (or \code{withLogErrors}).
+#'   `captureStackTraces` (or `withLogErrors`).
 #' @rdname stacktrace
 #' @export
 conditionStackTrace <- function(cond) {
@@ -607,8 +609,8 @@ conditionStackTrace <- function(cond) {
   invisible(cond)
 }
 
-#' @details The two functions \code{..stacktraceon..} and
-#'   \code{..stacktraceoff..} have no runtime behavior during normal execution;
+#' @details The two functions `..stacktraceon..` and
+#'   `..stacktraceoff..` have no runtime behavior during normal execution;
 #'   they exist only to create artifacts on the stack trace (sys.call()) that
 #'   instruct the stack trace pretty printer what parts of the stack trace are
 #'   interesting or not. The initial state is 1 and we walk from the outermost
@@ -624,4 +626,4 @@ conditionStackTrace <- function(cond) {
 #' @export
 ..stacktraceoff.. <- function(expr) expr
 
-..stacktracefloor.. <- function(expr) expr
\ No newline at end of file
+..stacktracefloor.. <- function(expr) expr
