@@ -3,37 +3,44 @@
 #' Create a select list that can be used to choose a single or multiple items
 #' from a list of values.
 #'
-#' By default, \code{selectInput()} and \code{selectizeInput()} use the
-#' JavaScript library \pkg{selectize.js}
-#' (\url{https://github.com/selectize/selectize.js}) instead of the basic
-#' select input element. To use the standard HTML select input element, use
-#' \code{selectInput()} with \code{selectize=FALSE}.
+#' By default, `selectInput()` and `selectizeInput()` use the JavaScript library
+#' \pkg{selectize.js} (<https://github.com/selectize/selectize.js>) instead of
+#' the basic select input element. To use the standard HTML select input
+#' element, use `selectInput()` with `selectize=FALSE`.
 #'
-#' In selectize mode, if the first element in \code{choices} has a value of
-#' \code{""}, its name will be treated as a placeholder prompt. For example:
-#' \code{selectInput("letter", "Letter", c("Choose one" = "", LETTERS))}
+#' In selectize mode, if the first element in `choices` has a value of `""`, its
+#' name will be treated as a placeholder prompt. For example:
+#' `selectInput("letter", "Letter", c("Choose one" = "", LETTERS))`
+#'
+#' **Performance note:** `selectInput()` and `selectizeInput()` can slow down
+#' significantly when thousands of choices are used; with legacy browsers like
+#' Internet Explorer, the user interface may hang for many seconds. For large
+#' numbers of choices, Shiny offers a "server-side selectize" option that
+#' massively improves performance and efficiency; see
+#' [this selectize article](https://shiny.rstudio.com/articles/selectize.html)
+#' on the Shiny Dev Center for details.
 #'
 #' @inheritParams textInput
 #' @param choices List of values to select from. If elements of the list are
-#'   named, then that name rather than the value is displayed to the user.
-#'   This can also be a named list whose elements are (either named or
-#'   unnamed) lists or vectors. If this is the case, the outermost names
-#'   will be used as the "optgroup" label for the elements in the respective
-#'   sublist. This allows you to group and label similar choices. See the
-#'   example section for a small demo of this feature.
-#' @param selected The initially selected value (or multiple values if
-#'   \code{multiple = TRUE}). If not specified then defaults to the first value
-#'   for single-select lists and no values for multiple select lists.
+#'   named, then that name --- rather than the value --- is displayed to the
+#'   user. It's also possible to group related inputs by providing a named list
+#'   whose elements are (either named or unnamed) lists, vectors, or factors. In
+#'   this case, the outermost names will be used as the group labels (leveraging
+#'   the `<optgroup>` HTML tag) for the elements in the respective sublist. See
+#'   the example section for a small demo of this feature.
+#' @param selected The initially selected value (or multiple values if `multiple
+#'   = TRUE`). If not specified then defaults to the first value for
+#'   single-select lists and no values for multiple select lists.
 #' @param multiple Is selection of multiple items allowed?
 #' @param selectize Whether to use \pkg{selectize.js} or not.
 #' @param size Number of items to show in the selection box; a larger number
-#'   will result in a taller box. Not compatible with \code{selectize=TRUE}.
-#'   Normally, when \code{multiple=FALSE}, a select input will be a drop-down
-#'   list, but when \code{size} is set, it will be a box instead.
+#'   will result in a taller box. Not compatible with `selectize=TRUE`.
+#'   Normally, when `multiple=FALSE`, a select input will be a drop-down list,
+#'   but when `size` is set, it will be a box instead.
 #' @return A select list control that can be added to a UI definition.
 #'
 #' @family input elements
-#' @seealso \code{\link{updateSelectInput}} \code{\link{varSelectInput}}
+#' @seealso [updateSelectInput()] [varSelectInput()]
 #'
 #' @examples
 #' ## Only run examples in interactive R sessions
@@ -55,7 +62,7 @@
 #'   }
 #' )
 #'
-#' # demoing optgroup support in the `choices` arg
+#' # demoing group support in the `choices` arg
 #' shinyApp(
 #'   ui = fluidPage(
 #'     selectInput("state", "Choose a state:",
@@ -72,6 +79,11 @@
 #'   }
 #' )
 #' }
+#'
+#' @section Server value: A vector of character strings, usually of length
+#'   1, with the value of the selected items. When `multiple=TRUE` and
+#'   nothing is selected, this value will be `NULL`.
+#'
 #' @export
 selectInput <- function(inputId, label, choices, selected = NULL,
   multiple = FALSE, selectize = TRUE, width = NULL,
@@ -96,7 +108,7 @@ selectInput <- function(inputId, label, choices, selected = NULL,
     id = inputId,
     class = if (!selectize) "form-control",
     size = size,
-    selectOptions(choices, selected)
+    selectOptions(choices, selected, inputId, selectize)
   )
   if (multiple)
     selectTag$attribs$multiple <- "multiple"
@@ -104,8 +116,8 @@ selectInput <- function(inputId, label, choices, selected = NULL,
   # return label and select tag
   res <- div(
     class = "form-group shiny-input-container",
-    style = if (!is.null(width)) paste0("width: ", validateCssUnit(width), ";"),
-    controlLabel(inputId, label),
+    style = css(width = validateCssUnit(width)),
+    shinyInputLabel(inputId, label),
     div(selectTag)
   )
 
@@ -121,16 +133,22 @@ firstChoice <- function(choices) {
 }
 
 # Create tags for each of the options; use <optgroup> if necessary.
-# This returns a HTML string instead of tags, because of the 'selected'
-# attribute.
-selectOptions <- function(choices, selected = NULL) {
+# This returns a HTML string instead of tags for performance reasons.
+selectOptions <- function(choices, selected = NULL, inputId, perfWarning = FALSE) {
+  if (length(choices) >= 1000) {
+    warning("The select input \"", inputId, "\" contains a large number of ",
+      "options; consider using server-side selectize for massively improved ",
+      "performance. See the Details section of the ?selectizeInput help topic.",
+      call. = FALSE)
+  }
+
   html <- mapply(choices, names(choices), FUN = function(choice, label) {
     if (is.list(choice)) {
       # If sub-list, create an optgroup and recurse into the sublist
       sprintf(
         '<optgroup label="%s">\n%s\n</optgroup>',
         htmlEscape(label, TRUE),
-        selectOptions(choice, selected)
+        selectOptions(choice, selected, inputId, perfWarning)
       )
 
     } else {
@@ -153,21 +171,21 @@ needOptgroup <- function(choices) {
 }
 
 #' @rdname selectInput
-#' @param ... Arguments passed to \code{selectInput()}.
+#' @param ... Arguments passed to `selectInput()`.
 #' @param options A list of options. See the documentation of \pkg{selectize.js}
-#'   for possible options (character option values inside \code{\link[base]{I}()} will
-#'   be treated as literal JavaScript code; see \code{\link{renderDataTable}()}
+#'   for possible options (character option values inside [base::I()] will
+#'   be treated as literal JavaScript code; see [renderDataTable()]
 #'   for details).
-#' @param width The width of the input, e.g. \code{'400px'}, or \code{'100\%'};
-#'   see \code{\link{validateCssUnit}}.
-#' @note The selectize input created from \code{selectizeInput()} allows
+#' @param width The width of the input, e.g. `'400px'`, or `'100%'`;
+#'   see [validateCssUnit()].
+#' @note The selectize input created from `selectizeInput()` allows
 #'   deletion of the selected option even in a single select input, which will
 #'   return an empty string as its value. This is the default behavior of
 #'   \pkg{selectize.js}. However, the selectize input created from
-#'   \code{selectInput(..., selectize = TRUE)} will ignore the empty string
+#'   `selectInput(..., selectize = TRUE)` will ignore the empty string
 #'   value when it is a single choice input and the empty string is not in the
-#'   \code{choices} argument. This is to keep compatibility with
-#'   \code{selectInput(..., selectize = FALSE)}.
+#'   `choices` argument. This is to keep compatibility with
+#'   `selectInput(..., selectize = FALSE)`.
 #' @export
 selectizeInput <- function(inputId, ..., options = NULL, width = NULL) {
   selectizeIt(
@@ -179,24 +197,30 @@ selectizeInput <- function(inputId, ..., options = NULL, width = NULL) {
 
 # given a select input and its id, selectize it
 selectizeIt <- function(inputId, select, options, nonempty = FALSE) {
+  if (length(options) == 0) {
+    # For NULL and empty unnamed list, replace with an empty named list, so that
+    # it will get translated to {} in JSON later on.
+    options <- empty_named_list()
+  }
+
+  # Make sure accessibility plugin is included
+  if (!('selectize-plugin-a11y' %in% options$plugins)) {
+    options$plugins <- c(options$plugins, list('selectize-plugin-a11y'))
+  }
+
   res <- checkAsIs(options)
 
-  selectizeDep <- htmlDependency(
-    "selectize", "0.11.2", c(href = "shared/selectize"),
-    stylesheet = "css/selectize.bootstrap3.css",
-    head = format(tagList(
-      HTML('<!--[if lt IE 9]>'),
-      tags$script(src = 'shared/selectize/js/es5-shim.min.js'),
-      HTML('<![endif]-->'),
-      tags$script(src = 'shared/selectize/js/selectize.min.js')
-    ))
-  )
+  deps <- list(selectizeDependency())
 
   if ('drag_drop' %in% options$plugins) {
-    selectizeDep <- list(selectizeDep, htmlDependency(
-      'jqueryui', '1.12.1', c(href = 'shared/jqueryui'),
-      script = 'jquery-ui.min.js'
-    ))
+    deps <- c(
+      deps,
+      list(htmlDependency(
+        'jqueryui', '1.12.1',
+        c(href = 'shared/jqueryui'),
+        script = 'jquery-ui.min.js'
+      ))
+    )
   }
 
   # Insert script on same level as <select> tag
@@ -206,18 +230,64 @@ selectizeIt <- function(inputId, select, options, nonempty = FALSE) {
       type = 'application/json',
       `data-for` = inputId, `data-nonempty` = if (nonempty) '',
       `data-eval` = if (length(res$eval)) HTML(toJSON(res$eval)),
-      if (length(res$options)) HTML(toJSON(res$options)) else '{}'
+      HTML(toJSON(res$options))
     )
   )
 
-  attachDependencies(select, selectizeDep)
+  attachDependencies(select, deps)
 }
 
 
+selectizeDependency <- function() {
+  bslib::bs_dependency_defer(selectizeDependencyFunc)
+}
 
+selectizeDependencyFunc <- function(theme) {
+  selectizeVersion <- "0.12.4"
+  if (!is_bs_theme(theme)) {
+    return(selectizeStaticDependency(selectizeVersion))
+  }
 
+  selectizeDir <- system.file(package = "shiny", "www/shared/selectize/")
+  stylesheet <- file.path(
+    selectizeDir, "scss",
+    if ("3" %in% bslib::theme_version(theme)) {
+      "selectize.bootstrap3.scss"
+    } else {
+      "selectize.bootstrap4.scss"
+    }
+  )
+  # It'd be cleaner to ship the JS in a separate, href-based,
+  # HTML dependency (which we currently do for other themable widgets),
+  # but DT, crosstalk, and maybe other pkgs include selectize JS/CSS
+  # in HTML dependency named selectize, so if we were to change that
+  # name, the JS/CSS would be loaded/included twice, which leads to
+  # strange issues, especially since we now include a 3rd party
+  # accessibility plugin https://github.com/rstudio/shiny/pull/3153
+  script <- file.path(
+    selectizeDir, c("js/selectize.min.js", "accessibility/js/selectize-plugin-a11y.min.js")
+  )
+  bslib::bs_dependency(
+    input = sass::sass_file(stylesheet),
+    theme = theme,
+    name = "selectize",
+    version = selectizeVersion,
+    cache_key_extra = shinyPackageVersion(),
+    .dep_args = list(script = script)
+  )
+}
 
-
+selectizeStaticDependency <- function(version) {
+  htmlDependency(
+    "selectize", version,
+    src = c(href = "shared/selectize"),
+    stylesheet = "css/selectize.bootstrap3.css",
+    script = c(
+      "js/selectize.min.js",
+      "accessibility/js/selectize-plugin-a11y.min.js"
+    )
+  )
+}
 
 
 #' Select variables from a data frame
@@ -225,30 +295,31 @@ selectizeIt <- function(inputId, select, options, nonempty = FALSE) {
 #' Create a select list that can be used to choose a single or multiple items
 #' from the column names of a data frame.
 #'
-#' The resulting server \code{input} value will be returned as:
-#' \itemize{
-#'   \item a symbol if \code{multiple = FALSE}.  The \code{input} value should be
-#'         used with rlang's \code{\link[rlang]{!!}}. For example,
-#'         \code{ggplot2::aes(!!input$variable)}.
-#'   \item a list of symbols if \code{multiple = TRUE}. The \code{input} value
-#'         should be used with rlang's \code{\link[rlang]{!!!}} to expand
-#'         the symbol list as individual arguments. For example,
-#'         \code{dplyr::select(mtcars, !!!input$variabls)} which is
-#'         equivalent to \code{dplyr::select(mtcars, !!input$variabls[[1]], !!input$variabls[[2]], ..., !!input$variabls[[length(input$variabls)]])}.
-#' }
-#'
-#' By default, \code{varSelectInput()} and \code{selectizeInput()} use the
+#' By default, `varSelectInput()` and `selectizeInput()` use the
 #' JavaScript library \pkg{selectize.js}
-#' (\url{https://github.com/selectize/selectize.js}) to instead of the basic
+#' (<https://github.com/selectize/selectize.js>) to instead of the basic
 #' select input element. To use the standard HTML select input element, use
-#' \code{selectInput()} with \code{selectize=FALSE}.
+#' `selectInput()` with `selectize=FALSE`.
 #'
 #' @inheritParams selectInput
-#' @param data A data frame. Used to retrieve the column names as choices for a \code{\link{selectInput}}
+#' @param data A data frame. Used to retrieve the column names as choices for a [selectInput()]
 #' @return A variable select list control that can be added to a UI definition.
 #'
 #' @family input elements
-#' @seealso \code{\link{updateSelectInput}}
+#' @seealso [updateSelectInput()]
+#'
+#' @section Server value:
+#' The resulting server `input` value will be returned as:
+#'
+#'  * A symbol if `multiple = FALSE`. The `input` value should be
+#'  used with rlang's [rlang::!!()]. For example,
+#'  `ggplot2::aes(!!input$variable)`.
+#'  * A list of symbols if `multiple = TRUE`. The `input` value
+#'  should be used with rlang's [rlang::!!!()] to expand
+#'  the symbol list as individual arguments. For example,
+#'  `dplyr::select(mtcars, !!!input$variabls)` which is
+#'  equivalent to `dplyr::select(mtcars, !!input$variabls[[1]], !!input$variabls[[2]], ..., !!input$variabls[[length(input$variabls)]])`.
+#'
 #' @examples
 #'
 #' ## Only run examples in interactive R sessions
@@ -321,21 +392,21 @@ varSelectInput <- function(
 
 
 #' @rdname varSelectInput
-#' @param ... Arguments passed to \code{varSelectInput()}.
+#' @param ... Arguments passed to `varSelectInput()`.
 #' @param options A list of options. See the documentation of \pkg{selectize.js}
-#'   for possible options (character option values inside \code{\link[base]{I}()} will
-#'   be treated as literal JavaScript code; see \code{\link{renderDataTable}()}
+#'   for possible options (character option values inside [base::I()] will
+#'   be treated as literal JavaScript code; see [renderDataTable()]
 #'   for details).
-#' @param width The width of the input, e.g. \code{'400px'}, or \code{'100\%'};
-#'   see \code{\link{validateCssUnit}}.
-#' @note The variable selectize input created from \code{varSelectizeInput()} allows
+#' @param width The width of the input, e.g. `'400px'`, or `'100%'`;
+#'   see [validateCssUnit()].
+#' @note The variable selectize input created from `varSelectizeInput()` allows
 #'   deletion of the selected option even in a single select input, which will
 #'   return an empty string as its value. This is the default behavior of
 #'   \pkg{selectize.js}. However, the selectize input created from
-#'   \code{selectInput(..., selectize = TRUE)} will ignore the empty string
+#'   `selectInput(..., selectize = TRUE)` will ignore the empty string
 #'   value when it is a single choice input and the empty string is not in the
-#'   \code{choices} argument. This is to keep compatibility with
-#'   \code{selectInput(..., selectize = FALSE)}.
+#'   `choices` argument. This is to keep compatibility with
+#'   `selectInput(..., selectize = FALSE)`.
 #' @export
 varSelectizeInput <- function(inputId, ..., options = NULL, width = NULL) {
   selectizeIt(
