@@ -31,11 +31,13 @@ Context <- R6Class(
     .flushCallbacks = list(),
     .domain = NULL,
     .pid = NULL,
+    .weak = NULL,
 
     initialize = function(
       domain, label='', type='other', prevId='',
       reactId = rLog$noReactId,
-      id = .getReactiveEnvironment()$nextId() # For dummy context
+      id = .getReactiveEnvironment()$nextId(), # For dummy context
+      weak = FALSE
     ) {
       id <<- id
       .label <<- label
@@ -43,6 +45,7 @@ Context <- R6Class(
       .pid <<- processId()
       .reactId <<- reactId
       .reactType <<- type
+      .weak <<- weak
       rLog$createContext(id, label, type, prevId, domain)
     },
     run = function(func) {
@@ -62,7 +65,7 @@ Context <- R6Class(
         that have been registered with onInvalidate()."
 
       if (!identical(.pid, processId())) {
-        stop("Reactive context was created in one process and invalidated from another")
+        rlang::abort("Reactive context was created in one process and invalidated from another.")
       }
 
       if (.invalidated)
@@ -84,7 +87,7 @@ Context <- R6Class(
         immediately."
 
       if (!identical(.pid, processId())) {
-        stop("Reactive context was created in one process and accessed from another")
+        rlang::abort("Reactive context was created in one process and accessed from another.")
       }
 
       if (.invalidated)
@@ -108,6 +111,9 @@ Context <- R6Class(
       lapply(.flushCallbacks, function(flushCallback) {
         flushCallback()
       })
+    },
+    isWeak = function() {
+      .weak
     }
   )
 )
@@ -134,9 +140,13 @@ ReactiveEnvironment <- R6Class(
         if (isTRUE(getOption('shiny.suppressMissingContextError'))) {
           return(getDummyContext())
         } else {
-          stop('Operation not allowed without an active reactive context. ',
-               '(You tried to do something that can only be done from inside a ',
-               'reactive expression or observer.)')
+          rlang::abort(c(
+            'Operation not allowed without an active reactive context.',
+            paste0(
+              'You tried to do something that can only be done from inside a ',
+              'reactive consumer.'
+            )
+          ))
         }
       }
       return(.currentContext)
@@ -196,7 +206,8 @@ getCurrentContext <- function() {
   .getReactiveEnvironment()$currentContext()
 }
 hasCurrentContext <- function() {
-  !is.null(.getReactiveEnvironment()$.currentContext)
+  !is.null(.getReactiveEnvironment()$.currentContext) ||
+    isTRUE(getOption("shiny.suppressMissingContextError"))
 }
 
 getDummyContext <- function() {
